'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _slicedToArray2 = require('babel-runtime/helpers/slicedToArray');

var _slicedToArray3 = _interopRequireDefault(_slicedToArray2);

var _typeof2 = require('babel-runtime/helpers/typeof');

var _typeof3 = _interopRequireDefault(_typeof2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _createClass2 = require('babel-runtime/helpers/createClass');

var _createClass3 = _interopRequireDefault(_createClass2);

exports.default = initAppManager;

var _slotFinder = require('slot-finder');

var _slotFinder2 = _interopRequireDefault(_slotFinder);

var _path = require('./utils/path');

var _timers = require('./utils/timers');

var _constants = require('./constants');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function initAppManager(w) {
  var _class, _temp;

  return _temp = _class = function () {
    (0, _createClass3.default)(AppManager, null, [{
      key: '_getBrowserPath',
      value: function _getBrowserPath() {
        return w.location.pathname;
      }
    }, {
      key: '_querySelector',
      value: function _querySelector(querySelector) {
        return w.document.querySelector(querySelector);
      }
    }, {
      key: '_goToPath',
      value: function _goToPath() {
        w.location.href = AppManager._getBrowserPath(); // eslint-disable-line no-param-reassign
      }
    }]);

    function AppManager(config, events) {
      var _this = this;

      var options = arguments.length > 2 && arguments[2] !== undefined ? arguments[2] : {};
      (0, _classCallCheck3.default)(this, AppManager);
      this._status = this.constructor.statuses.UNINITIALISED;
      this._currentAppName = null;
      this._resolvedFragments = {};

      this.emit = function (eventTitle, data) {
        if (_this._events && typeof _this._events.emit === 'function') {
          _this._events.emit(eventTitle, data);
        }
      };

      this.on = function (eventTitle, listener) {
        if (_this._events && typeof _this._events.on === 'function') {
          _this._events.on(eventTitle, listener);
        }
      };

      this.removeListener = function (eventTitle, listener) {
        if (_this._events && typeof _this._events.removeListener === 'function') {
          _this._events.removeListener(eventTitle, listener);
        }
      };

      this._setStatus = function (newStatus) {
        if (_this._status !== newStatus) {
          _this._status = newStatus;
          _this.emit(_this.constructor.eventTitles.STATUS_CHANGE, newStatus);
        }
      };

      this._onError = function (title, id, err) {
        _this._setStatus(_this.constructor.statuses.ERROR);
        _this.emit(_this.constructor.eventTitles.ERROR, { title: title, id: id, err: err });
      };

      this._getAppNameFromPath = function () {
        var path = _this.constructor._getBrowserPath();

        if (!path || typeof path !== 'string') {
          _this._onError('get_app_from_path.no_path');

          return null;
        }

        return (0, _path.getAppNameFromPath)(_this._evaluatedPaths, path);
      };

      this._getPathParams = function (appName) {
        var path = _this.constructor._getBrowserPath();

        if (!path || typeof path !== 'string') {
          _this._onError('get_path_params.no_path');

          return null;
        }

        var evaluatedPath = _this._evaluatedPaths[appName];

        return (0, _path.evalPathParams)(evaluatedPath, path);
      };

      this._getSlotElement = function () {
        var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(slot) {
          var querySelector, element;
          return _regenerator2.default.wrap(function _callee$(_context) {
            while (1) {
              switch (_context.prev = _context.next) {
                case 0:
                  querySelector = slot.querySelector;

                  if (querySelector) {
                    _context.next = 3;
                    break;
                  }

                  return _context.abrupt('return', null);

                case 3:
                  _context.next = 5;
                  return (0, _timers.retry)(function () {
                    return _this.constructor._querySelector(querySelector);
                  }, _constants.DOM_CHECK_INTERVAL, _this._options.importTimeout);

                case 5:
                  element = _context.sent;

                  if (element) {
                    _context.next = 8;
                    break;
                  }

                  throw new Error('Element not found.');

                case 8:
                  return _context.abrupt('return', element);

                case 9:
                case 'end':
                  return _context.stop();
              }
            }
          }, _callee, _this);
        }));

        return function (_x2) {
          return _ref.apply(this, arguments);
        };
      }();

      this._loadFragmentScript = function () {
        var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(fragmentName, appName) {
          var fragment, state, _script;

          return _regenerator2.default.wrap(function _callee2$(_context2) {
            while (1) {
              switch (_context2.prev = _context2.next) {
                case 0:
                  fragment = _this._fragments[fragmentName];
                  state = _this._getState(appName);

                  if (fragment) {
                    _context2.next = 5;
                    break;
                  }

                  _this._onError('load_fragment_script.invalid_fragment', fragmentName);

                  return _context2.abrupt('return', null);

                case 5:
                  if (!(typeof fragment.loadScript !== 'function')) {
                    _context2.next = 8;
                    break;
                  }

                  _this._onError('load_fragment_script.not_function', fragmentName);

                  return _context2.abrupt('return', null);

                case 8:
                  _context2.prev = 8;
                  _context2.next = 11;
                  return _promise2.default.race([fragment.loadScript(state), (0, _timers.delay)(_this._options.importTimeout)]);

                case 11:
                  _script = _context2.sent;

                  if (_script) {
                    _context2.next = 15;
                    break;
                  }

                  _this._onError('load_fragment_script.no_script', fragmentName);

                  return _context2.abrupt('return', null);

                case 15:
                  if ('version' in _script) {
                    _context2.next = 18;
                    break;
                  }

                  _this._onError('load_fragment_script.invalid_script', fragmentName);

                  return _context2.abrupt('return', null);

                case 18:
                  return _context2.abrupt('return', _script);

                case 21:
                  _context2.prev = 21;
                  _context2.t0 = _context2['catch'](8);

                  _this._onError('load_fragment_script.load', fragmentName, _context2.t0);

                  return _context2.abrupt('return', null);

                case 25:
                case 'end':
                  return _context2.stop();
              }
            }
          }, _callee2, _this, [[8, 21]]);
        }));

        return function (_x3, _x4) {
          return _ref2.apply(this, arguments);
        };
      }();

      this._resolveFragment = function () {
        var _ref3 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee3(slotName, fragmentName, appName) {
          var script, resolvedFragment;
          return _regenerator2.default.wrap(function _callee3$(_context3) {
            while (1) {
              switch (_context3.prev = _context3.next) {
                case 0:
                  if (!_this._resolvedFragments[fragmentName]) {
                    _context3.next = 2;
                    break;
                  }

                  return _context3.abrupt('return', _this._resolvedFragments[fragmentName]);

                case 2:
                  _context3.next = 4;
                  return _this._loadFragmentScript(fragmentName, appName);

                case 4:
                  script = _context3.sent;

                  if (script) {
                    _context3.next = 7;
                    break;
                  }

                  return _context3.abrupt('return', null);

                case 7:
                  resolvedFragment = { script: script, slot: slotName, name: fragmentName };


                  _this._resolvedFragments[fragmentName] = resolvedFragment;

                  return _context3.abrupt('return', resolvedFragment);

                case 10:
                case 'end':
                  return _context3.stop();
              }
            }
          }, _callee3, _this);
        }));

        return function (_x5, _x6, _x7) {
          return _ref3.apply(this, arguments);
        };
      }();

      this._resolvePageFragments = function () {
        var _ref4 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee4(appName) {
          var app, appFragments, emptySlots, pageFragments, resolveFragmentPromises;
          return _regenerator2.default.wrap(function _callee4$(_context4) {
            while (1) {
              switch (_context4.prev = _context4.next) {
                case 0:
                  app = _this._apps[appName];

                  if (Array.isArray(app.fragments)) {
                    _context4.next = 4;
                    break;
                  }

                  _this._onError('resolve_page_fragments.invalid_app', appName);

                  return _context4.abrupt('return', null);

                case 4:
                  appFragments = app.fragments.map(function (fragmentName) {
                    return _this._fragments[fragmentName];
                  });
                  emptySlots = (0, _keys2.default)(_this._slots).reduce(function (empty, slotName) {
                    return (0, _assign2.default)(empty, (0, _defineProperty3.default)({}, slotName, null));
                  }, {});
                  pageFragments = (0, _slotFinder2.default)(emptySlots, appFragments);

                  if (!(!pageFragments || (typeof pageFragments === 'undefined' ? 'undefined' : (0, _typeof3.default)(pageFragments)) !== 'object')) {
                    _context4.next = 10;
                    break;
                  }

                  _this._onError('resolve_page_fragments.get', appName);

                  return _context4.abrupt('return', null);

                case 10:
                  resolveFragmentPromises = (0, _keys2.default)(pageFragments).filter(function (slotName) {
                    var fragmentName = pageFragments[slotName];
                    return fragmentName && _this._fragments[fragmentName].managed;
                  }).map(function (slotName) {
                    return _this._resolveFragment(slotName, pageFragments[slotName], appName);
                  });
                  return _context4.abrupt('return', _promise2.default.all(resolveFragmentPromises));

                case 12:
                case 'end':
                  return _context4.stop();
              }
            }
          }, _callee4, _this);
        }));

        return function (_x8) {
          return _ref4.apply(this, arguments);
        };
      }();

      this._getState = function (appName) {
        var eventTitle = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
        return {
          status: _this._status,
          event: eventTitle,
          path: _this.constructor._getBrowserPath(),
          params: _this._getPathParams(appName) || {},
          prevApp: _this._currentAppName ? _this._apps[_this._currentAppName] : null,
          app: _this._apps[appName]
        };
      };

      this._hydrate = function (appName) {
        return function () {
          var _ref5 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee5(resolvedFragment) {
            var script, name, slotName, slot, slotElement, app, state;
            return _regenerator2.default.wrap(function _callee5$(_context5) {
              while (1) {
                switch (_context5.prev = _context5.next) {
                  case 0:
                    script = resolvedFragment.script, name = resolvedFragment.name, slotName = resolvedFragment.slot;
                    slot = _this._slots[slotName];

                    if (slot) {
                      _context5.next = 5;
                      break;
                    }

                    _this._onError('hydrate.no_slot', slotName);

                    return _context5.abrupt('return', false);

                  case 5:
                    slotElement = void 0;
                    _context5.prev = 6;
                    _context5.next = 9;
                    return _this._getSlotElement(slot);

                  case 9:
                    slotElement = _context5.sent;
                    _context5.next = 16;
                    break;

                  case 12:
                    _context5.prev = 12;
                    _context5.t0 = _context5['catch'](6);

                    _this._onError('hydrate.no_element', slot.name);

                    return _context5.abrupt('return', false);

                  case 16:
                    if (slotElement) {
                      _context5.next = 18;
                      break;
                    }

                    return _context5.abrupt('return', true);

                  case 18:
                    app = _this._apps[appName];
                    state = _this._getState(appName);
                    _context5.prev = 20;

                    if (!(script.version === 3)) {
                      _context5.next = 26;
                      break;
                    }

                    _context5.next = 24;
                    return script.hydrate(slotElement, w.history, app);

                  case 24:
                    _context5.next = 44;
                    break;

                  case 26:
                    if (!(script.version === 4)) {
                      _context5.next = 36;
                      break;
                    }

                    if (!(typeof script.hydrate === 'function')) {
                      _context5.next = 32;
                      break;
                    }

                    _context5.next = 30;
                    return script.hydrate(slotElement, app);

                  case 30:
                    _context5.next = 34;
                    break;

                  case 32:
                    _context5.next = 34;
                    return script.mount(slotElement, _constants.eventTitles.HISTORY_REPLACE_STATE, app);

                  case 34:
                    _context5.next = 44;
                    break;

                  case 36:
                    if (!(typeof script.hydrate === 'function')) {
                      _context5.next = 41;
                      break;
                    }

                    _context5.next = 39;
                    return script.hydrate(slotElement, state);

                  case 39:
                    _context5.next = 44;
                    break;

                  case 41:
                    if (!(typeof script.render === 'function')) {
                      _context5.next = 44;
                      break;
                    }

                    _context5.next = 44;
                    return script.render(slotElement, state);

                  case 44:
                    return _context5.abrupt('return', true);

                  case 47:
                    _context5.prev = 47;
                    _context5.t1 = _context5['catch'](20);

                    _this._onError('hydrate.run_script', name, _context5.t1);

                    return _context5.abrupt('return', false);

                  case 51:
                  case 'end':
                    return _context5.stop();
                }
              }
            }, _callee5, _this, [[6, 12], [20, 47]]);
          }));

          return function (_x10) {
            return _ref5.apply(this, arguments);
          };
        }();
      };

      this._unmount = function (appName, eventTitle) {
        return function () {
          var _ref6 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee6(resolvedFragment) {
            var script, name, slotName, _slot, slotElement, app, state;

            return _regenerator2.default.wrap(function _callee6$(_context6) {
              while (1) {
                switch (_context6.prev = _context6.next) {
                  case 0:
                    script = resolvedFragment.script, name = resolvedFragment.name, slotName = resolvedFragment.slot;
                    _context6.prev = 1;
                    _slot = _this._slots[slotName];

                    if (_slot) {
                      _context6.next = 6;
                      break;
                    }

                    _this._onError('unmount.no_slot', slotName);

                    return _context6.abrupt('return', false);

                  case 6:
                    slotElement = void 0;
                    _context6.prev = 7;
                    _context6.next = 10;
                    return _this._getSlotElement(_slot);

                  case 10:
                    slotElement = _context6.sent;
                    _context6.next = 17;
                    break;

                  case 13:
                    _context6.prev = 13;
                    _context6.t0 = _context6['catch'](7);

                    _this._onError('unmount.no_element', _slot.name);

                    return _context6.abrupt('return', false);

                  case 17:
                    if (slotElement) {
                      _context6.next = 19;
                      break;
                    }

                    return _context6.abrupt('return', true);

                  case 19:
                    app = _this._apps[appName];
                    state = _this._getState(appName, eventTitle);

                    if (!(script.version === 3)) {
                      _context6.next = 26;
                      break;
                    }

                    _context6.next = 24;
                    return script.unmount(slotElement, w.history, app);

                  case 24:
                    _context6.next = 34;
                    break;

                  case 26:
                    if (!(script.version === 4)) {
                      _context6.next = 31;
                      break;
                    }

                    _context6.next = 29;
                    return script.unmount(slotElement, eventTitle, app);

                  case 29:
                    _context6.next = 34;
                    break;

                  case 31:
                    if (!(typeof script.unmount === 'function')) {
                      _context6.next = 34;
                      break;
                    }

                    _context6.next = 34;
                    return script.unmount(slotElement, state);

                  case 34:
                    return _context6.abrupt('return', true);

                  case 37:
                    _context6.prev = 37;
                    _context6.t1 = _context6['catch'](1);

                    _this._onError('unmount.error', name || appName, _context6.t1);

                    return _context6.abrupt('return', false);

                  case 41:
                  case 'end':
                    return _context6.stop();
                }
              }
            }, _callee6, _this, [[1, 37], [7, 13]]);
          }));

          return function (_x11) {
            return _ref6.apply(this, arguments);
          };
        }();
      };

      this._mount = function (appName, eventTitle) {
        return function () {
          var _ref7 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee7(resolvedFragment) {
            var script, name, slotName, _slot2, slotElement, app, state;

            return _regenerator2.default.wrap(function _callee7$(_context7) {
              while (1) {
                switch (_context7.prev = _context7.next) {
                  case 0:
                    script = resolvedFragment.script, name = resolvedFragment.name, slotName = resolvedFragment.slot;
                    _context7.prev = 1;
                    _slot2 = _this._slots[slotName];

                    if (_slot2) {
                      _context7.next = 6;
                      break;
                    }

                    _this._onError('mount.no_slot', slotName);

                    return _context7.abrupt('return', false);

                  case 6:
                    slotElement = void 0;
                    _context7.prev = 7;
                    _context7.next = 10;
                    return _this._getSlotElement(_slot2);

                  case 10:
                    slotElement = _context7.sent;
                    _context7.next = 17;
                    break;

                  case 13:
                    _context7.prev = 13;
                    _context7.t0 = _context7['catch'](7);

                    _this._onError('mount.no_element', _slot2.name);

                    return _context7.abrupt('return', false);

                  case 17:
                    if (slotElement) {
                      _context7.next = 19;
                      break;
                    }

                    return _context7.abrupt('return', true);

                  case 19:
                    app = _this._apps[appName];
                    state = _this._getState(appName, eventTitle);

                    if (!(script.version === 3)) {
                      _context7.next = 26;
                      break;
                    }

                    _context7.next = 24;
                    return script.mount(slotElement, w.history, app);

                  case 24:
                    _context7.next = 34;
                    break;

                  case 26:
                    if (!(script.version === 4)) {
                      _context7.next = 31;
                      break;
                    }

                    _context7.next = 29;
                    return script.mount(slotElement, eventTitle, app);

                  case 29:
                    _context7.next = 34;
                    break;

                  case 31:
                    if (!(typeof script.render === 'function')) {
                      _context7.next = 34;
                      break;
                    }

                    _context7.next = 34;
                    return script.render(slotElement, state);

                  case 34:
                    return _context7.abrupt('return', true);

                  case 37:
                    _context7.prev = 37;
                    _context7.t1 = _context7['catch'](1);

                    _this._onError('mount.error', name || appName, _context7.t1);

                    return _context7.abrupt('return', false);

                  case 41:
                  case 'end':
                    return _context7.stop();
                }
              }
            }, _callee7, _this, [[1, 37], [7, 13]]);
          }));

          return function (_x12) {
            return _ref7.apply(this, arguments);
          };
        }();
      };

      this._onStateChange = function (appName, eventTitle) {
        return function () {
          var _ref8 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee8(resolvedFragment) {
            var script, name, app, state;
            return _regenerator2.default.wrap(function _callee8$(_context8) {
              while (1) {
                switch (_context8.prev = _context8.next) {
                  case 0:
                    script = resolvedFragment.script, name = resolvedFragment.name;
                    _context8.prev = 1;
                    app = _this._apps[appName];
                    state = _this._getState(appName, eventTitle);

                    if (!(script.version === 3)) {
                      _context8.next = 9;
                      break;
                    }

                    _context8.next = 7;
                    return script.onStateChange(w.history, app);

                  case 7:
                    _context8.next = 18;
                    break;

                  case 9:
                    if (!(script.version === 4)) {
                      _context8.next = 15;
                      break;
                    }

                    if (!(typeof script.onStateChange === 'function')) {
                      _context8.next = 13;
                      break;
                    }

                    _context8.next = 13;
                    return script.onStateChange(eventTitle, app);

                  case 13:
                    _context8.next = 18;
                    break;

                  case 15:
                    if (!(typeof script.onStateChange === 'function')) {
                      _context8.next = 18;
                      break;
                    }

                    _context8.next = 18;
                    return script.onStateChange(state);

                  case 18:
                    return _context8.abrupt('return', true);

                  case 21:
                    _context8.prev = 21;
                    _context8.t0 = _context8['catch'](1);

                    _this._onError('on_state_change.error', name || appName, _context8.t0);

                    return _context8.abrupt('return', false);

                  case 25:
                  case 'end':
                    return _context8.stop();
                }
              }
            }, _callee8, _this, [[1, 21]]);
          }));

          return function (_x13) {
            return _ref8.apply(this, arguments);
          };
        }();
      };

      this._onUpdateStatus = function (appName, status) {
        return function () {
          var _ref9 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee9(resolvedFragment) {
            var script, name, app, state;
            return _regenerator2.default.wrap(function _callee9$(_context9) {
              while (1) {
                switch (_context9.prev = _context9.next) {
                  case 0:
                    script = resolvedFragment.script, name = resolvedFragment.name;

                    if (!(script.version === 3)) {
                      _context9.next = 3;
                      break;
                    }

                    return _context9.abrupt('return', true);

                  case 3:
                    _context9.prev = 3;
                    app = _this._apps[appName];
                    state = _this._getState(appName);

                    if (!(script.version === 4)) {
                      _context9.next = 12;
                      break;
                    }

                    if (!(typeof script.onUpdateStatus === 'function')) {
                      _context9.next = 10;
                      break;
                    }

                    _context9.next = 10;
                    return script.onUpdateStatus(status, app);

                  case 10:
                    _context9.next = 15;
                    break;

                  case 12:
                    if (!(typeof script.onUpdateStatus === 'function')) {
                      _context9.next = 15;
                      break;
                    }

                    _context9.next = 15;
                    return script.onUpdateStatus(state);

                  case 15:
                    return _context9.abrupt('return', true);

                  case 18:
                    _context9.prev = 18;
                    _context9.t0 = _context9['catch'](3);

                    _this._onError('on_update_status.error', name || appName, _context9.t0);

                    return _context9.abrupt('return', false);

                  case 22:
                  case 'end':
                    return _context9.stop();
                }
              }
            }, _callee9, _this, [[3, 18]]);
          }));

          return function (_x14) {
            return _ref9.apply(this, arguments);
          };
        }();
      };

      this._hydratePage = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee10() {
        var currentAppName, resolvedFragments, hydratePromises, hydrateSuccess;
        return _regenerator2.default.wrap(function _callee10$(_context10) {
          while (1) {
            switch (_context10.prev = _context10.next) {
              case 0:
                currentAppName = _this._currentAppName;

                if (currentAppName) {
                  _context10.next = 4;
                  break;
                }

                _this._onError('hydrate_page.missing_route');

                return _context10.abrupt('return', false);

              case 4:
                _context10.prev = 4;
                _context10.next = 7;
                return _this._resolvePageFragments(currentAppName);

              case 7:
                resolvedFragments = _context10.sent;

                if (!(!Array.isArray(resolvedFragments) || !resolvedFragments.every(Boolean))) {
                  _context10.next = 10;
                  break;
                }

                return _context10.abrupt('return', false);

              case 10:
                hydratePromises = resolvedFragments.map(_this._hydrate(currentAppName));
                _context10.next = 13;
                return _promise2.default.all(hydratePromises);

              case 13:
                hydrateSuccess = _context10.sent;
                return _context10.abrupt('return', hydrateSuccess.every(Boolean));

              case 17:
                _context10.prev = 17;
                _context10.t0 = _context10['catch'](4);

                _this._onError('hydrate_page.get_scripts', currentAppName, _context10.t0);

                return _context10.abrupt('return', false);

              case 21:
              case 'end':
                return _context10.stop();
            }
          }
        }, _callee10, _this, [[4, 17]]);
      }));

      this._updatePage = function () {
        var _ref11 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee11(newAppName, eventTitle) {
          var oldAppName, _ref12, _ref13, oldResolvedFragments, newResolvedFragments, oldFragmentNames, newFragmentNames, unmountPromises, mountPromises, onStateChangePromises, unmountSuccess, mountSuccess, onStateChangeSuccess;

          return _regenerator2.default.wrap(function _callee11$(_context11) {
            while (1) {
              switch (_context11.prev = _context11.next) {
                case 0:
                  oldAppName = _this._currentAppName;

                  if (oldAppName) {
                    _context11.next = 4;
                    break;
                  }

                  _this._onError('update_page.no_current_app');

                  return _context11.abrupt('return', false);

                case 4:
                  _context11.next = 6;
                  return _promise2.default.all([_this._resolvePageFragments(oldAppName), _this._resolvePageFragments(newAppName)]);

                case 6:
                  _ref12 = _context11.sent;
                  _ref13 = (0, _slicedToArray3.default)(_ref12, 2);
                  oldResolvedFragments = _ref13[0];
                  newResolvedFragments = _ref13[1];

                  if (!(!Array.isArray(oldResolvedFragments) || !Array.isArray(newResolvedFragments) || !oldResolvedFragments.every(Boolean) || !newResolvedFragments.every(Boolean))) {
                    _context11.next = 13;
                    break;
                  }

                  _this._onError('update_page.invalid_fragments');

                  return _context11.abrupt('return', false);

                case 13:
                  oldFragmentNames = oldResolvedFragments.map(function (_ref14) {
                    var name = _ref14.name;
                    return name;
                  });
                  newFragmentNames = newResolvedFragments.map(function (_ref15) {
                    var name = _ref15.name;
                    return name;
                  });
                  unmountPromises = oldResolvedFragments.filter(function (resolvedFragment) {
                    return newFragmentNames.indexOf(resolvedFragment.name) === -1;
                  }).map(_this._unmount(oldAppName, eventTitle));
                  mountPromises = newResolvedFragments.filter(function (resolvedFragment) {
                    return oldFragmentNames.indexOf(resolvedFragment.name) === -1;
                  }).map(_this._mount(newAppName, eventTitle));
                  onStateChangePromises = oldResolvedFragments.filter(function (resolvedFragment) {
                    return newFragmentNames.indexOf(resolvedFragment.name) > -1;
                  }).map(_this._onStateChange(newAppName, eventTitle));
                  _context11.next = 20;
                  return _promise2.default.all(unmountPromises);

                case 20:
                  unmountSuccess = _context11.sent;

                  if (unmountSuccess.every(Boolean)) {
                    _context11.next = 23;
                    break;
                  }

                  return _context11.abrupt('return', false);

                case 23:
                  _context11.next = 25;
                  return _promise2.default.all(mountPromises);

                case 25:
                  mountSuccess = _context11.sent;

                  if (mountSuccess.every(Boolean)) {
                    _context11.next = 28;
                    break;
                  }

                  return _context11.abrupt('return', false);

                case 28:
                  _context11.next = 30;
                  return _promise2.default.all(onStateChangePromises);

                case 30:
                  onStateChangeSuccess = _context11.sent;
                  return _context11.abrupt('return', onStateChangeSuccess.every(Boolean));

                case 32:
                case 'end':
                  return _context11.stop();
              }
            }
          }, _callee11, _this);
        }));

        return function (_x15, _x16) {
          return _ref11.apply(this, arguments);
        };
      }();

      this._handleStateChange = function () {
        var _ref16 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee12(eventTitle) {
          var newAppName, updatePageSuccess;
          return _regenerator2.default.wrap(function _callee12$(_context12) {
            while (1) {
              switch (_context12.prev = _context12.next) {
                case 0:
                  if (!(_this._status === _this.constructor.statuses.ERROR)) {
                    _context12.next = 3;
                    break;
                  }

                  _this.constructor._goToPath();

                  return _context12.abrupt('return', false);

                case 3:
                  if (!(_this._status === _this.constructor.statuses.UNINITIALISED)) {
                    _context12.next = 6;
                    break;
                  }

                  _this._onError('handle_state_change.app_not_ready');

                  return _context12.abrupt('return', false);

                case 6:
                  if (!(typeof eventTitle !== 'string')) {
                    _context12.next = 9;
                    break;
                  }

                  _this._onError('handle_state_change.invalid_event_title');

                  return _context12.abrupt('return', false);

                case 9:

                  _this._setStatus(_this.constructor.statuses.LOADING);

                  newAppName = _this._getAppNameFromPath();

                  if (newAppName) {
                    _context12.next = 15;
                    break;
                  }

                  _this._onError('handle_state_change.app_not_found');
                  _this.constructor._goToPath();

                  return _context12.abrupt('return', false);

                case 15:
                  _context12.prev = 15;
                  _context12.next = 18;
                  return _this._updatePage(newAppName, eventTitle);

                case 18:
                  updatePageSuccess = _context12.sent;

                  if (updatePageSuccess) {
                    _context12.next = 23;
                    break;
                  }

                  _this._setStatus(_this.constructor.statuses.ERROR);
                  _this.constructor._goToPath();

                  return _context12.abrupt('return', false);

                case 23:

                  _this._currentAppName = newAppName;
                  _this._setStatus(_this.constructor.statuses.DEFAULT);
                  _this.emit(_this.constructor.eventTitles.STATE_CHANGE_COMPLETE);

                  return _context12.abrupt('return', true);

                case 29:
                  _context12.prev = 29;
                  _context12.t0 = _context12['catch'](15);

                  _this._onError('handle_state_change', newAppName, _context12.t0);
                  _this.constructor._goToPath();

                  return _context12.abrupt('return', false);

                case 34:
                case 'end':
                  return _context12.stop();
              }
            }
          }, _callee12, _this, [[15, 29]]);
        }));

        return function (_x17) {
          return _ref16.apply(this, arguments);
        };
      }();

      this._handleUpdateStatus = function () {
        var _ref17 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee13(status) {
          var currentAppName, resolvedFragments, updateStatusPromises, updateStatusSuccess;
          return _regenerator2.default.wrap(function _callee13$(_context13) {
            while (1) {
              switch (_context13.prev = _context13.next) {
                case 0:
                  currentAppName = _this._currentAppName;

                  if (currentAppName) {
                    _context13.next = 3;
                    break;
                  }

                  return _context13.abrupt('return', false);

                case 3:
                  if (!(typeof status !== 'string')) {
                    _context13.next = 5;
                    break;
                  }

                  return _context13.abrupt('return', false);

                case 5:
                  _context13.prev = 5;
                  _context13.next = 8;
                  return _this._resolvePageFragments(currentAppName);

                case 8:
                  resolvedFragments = _context13.sent;

                  if (!(!Array.isArray(resolvedFragments) || !resolvedFragments.every(Boolean))) {
                    _context13.next = 11;
                    break;
                  }

                  return _context13.abrupt('return', false);

                case 11:
                  updateStatusPromises = resolvedFragments.map(_this._onUpdateStatus(currentAppName, status));
                  _context13.next = 14;
                  return _promise2.default.all(updateStatusPromises);

                case 14:
                  updateStatusSuccess = _context13.sent;
                  return _context13.abrupt('return', updateStatusSuccess.every(Boolean));

                case 18:
                  _context13.prev = 18;
                  _context13.t0 = _context13['catch'](5);

                  _this._onError('handle_update_status', currentAppName, _context13.t0);

                  return _context13.abrupt('return', false);

                case 22:
                case 'end':
                  return _context13.stop();
              }
            }
          }, _callee13, _this, [[5, 18]]);
        }));

        return function (_x18) {
          return _ref17.apply(this, arguments);
        };
      }();

      this.init = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee14() {
        var currentAppName, hydrateSuccess;
        return _regenerator2.default.wrap(function _callee14$(_context14) {
          while (1) {
            switch (_context14.prev = _context14.next) {
              case 0:
                _this._shimHistory();

                _this._setStatus(_this.constructor.statuses.LOADING);

                currentAppName = _this._getAppNameFromPath();

                if (currentAppName) {
                  _context14.next = 6;
                  break;
                }

                _this._onError('init.no_route');

                return _context14.abrupt('return', false);

              case 6:

                _this._currentAppName = currentAppName;

                _context14.next = 9;
                return _this._hydratePage();

              case 9:
                hydrateSuccess = _context14.sent;

                if (hydrateSuccess) {
                  _context14.next = 13;
                  break;
                }

                _this._setStatus(_this.constructor.statuses.ERROR);

                return _context14.abrupt('return', false);

              case 13:

                _this._setStatus(_this.constructor.statuses.DEFAULT);

                return _context14.abrupt('return', true);

              case 15:
              case 'end':
                return _context14.stop();
            }
          }
        }, _callee14, _this);
      }));

      if (!config) {
        throw new Error('Config not specified. Aborting.');
      }

      if (!w.document || !w.location || !w.history) {
        throw new Error('Invalid window object. Aborting.');
      }

      var apps = config.apps,
          slots = config.slots,
          fragments = config.fragments;


      this._apps = apps;
      this._slots = slots;
      this._fragments = fragments;
      this._events = events;
      this._options = options;
      this._evaluatedPaths = (0, _path.evaluatePaths)(this._apps);
    }

    (0, _createClass3.default)(AppManager, [{
      key: '_shimHistory',
      value: function _shimHistory() {
        var _this2 = this;

        /* eslint-disable no-param-reassign */

        var onpopstate = w.onpopstate,
            onbeforeunload = w.onbeforeunload;
        var _w$history = w.history,
            pushState = _w$history.pushState,
            replaceState = _w$history.replaceState;


        w.history.pushState = function () {
          for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
            args[_key] = arguments[_key];
          }

          if (typeof pushState === 'function') {
            pushState.apply(w.history, args);
          }

          _this2.emit(_constants.eventTitles.HISTORY_PUSH_STATE);
          _this2.emit(_constants.eventTitles.HISTORY_STATE_CHANGE, _constants.eventTitles.HISTORY_PUSH_STATE);
        };

        w.history.replaceState = function () {
          for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
            args[_key2] = arguments[_key2];
          }

          if (typeof replaceState === 'function') {
            replaceState.apply(w.history, args);
          }

          _this2.emit(_constants.eventTitles.HISTORY_REPLACE_STATE);
          _this2.emit(_constants.eventTitles.HISTORY_STATE_CHANGE, _constants.eventTitles.HISTORY_REPLACE_STATE);
        };

        // $FlowFixMe
        w.history.getState = function () {
          return {
            data: w.history.state,
            title: w.document.title,
            url: w.location.href,
            hash: '' + w.location.pathname + (w.location.query || '')
          };
        };

        w.onpopstate = function () {
          for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
            args[_key3] = arguments[_key3];
          }

          if (typeof onpopstate === 'function') {
            onpopstate.apply(w.history, args);
          }

          _this2.emit(_constants.eventTitles.HISTORY_POP_STATE);
          _this2.emit(_constants.eventTitles.HISTORY_STATE_CHANGE, _constants.eventTitles.HISTORY_POP_STATE);
        };

        w.onbeforeunload = function () {
          for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
            args[_key4] = arguments[_key4];
          }

          if (typeof onbeforeunload === 'function') {
            onbeforeunload.apply(w.history, args);
          }

          _this2.removeListener(_this2.constructor.eventTitles.HISTORY_STATE_CHANGE, _this2._handleStateChange);
          _this2.removeListener(_this2.constructor.eventTitles.STATUS_CHANGE, _this2._handleUpdateStatus);

          w.onpopstate = onpopstate;
          w.history.pushState = pushState;
          w.history.replaceState = replaceState;
        };

        this.on(this.constructor.eventTitles.HISTORY_STATE_CHANGE, this._handleStateChange);
        this.on(this.constructor.eventTitles.STATUS_CHANGE, this._handleUpdateStatus);
      }
    }]);
    return AppManager;
  }(), _class.statuses = _constants.statuses, _class.eventTitles = _constants.eventTitles, _temp;
}