'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _promise = require('babel-runtime/core-js/promise');

var _promise2 = _interopRequireDefault(_promise);

var _regenerator = require('babel-runtime/regenerator');

var _regenerator2 = _interopRequireDefault(_regenerator);

var _toConsumableArray2 = require('babel-runtime/helpers/toConsumableArray');

var _toConsumableArray3 = _interopRequireDefault(_toConsumableArray2);

var _asyncToGenerator2 = require('babel-runtime/helpers/asyncToGenerator');

var _asyncToGenerator3 = _interopRequireDefault(_asyncToGenerator2);

var _defineProperty2 = require('babel-runtime/helpers/defineProperty');

var _defineProperty3 = _interopRequireDefault(_defineProperty2);

var _assign = require('babel-runtime/core-js/object/assign');

var _assign2 = _interopRequireDefault(_assign);

var _keys = require('babel-runtime/core-js/object/keys');

var _keys2 = _interopRequireDefault(_keys);

var _classCallCheck2 = require('babel-runtime/helpers/classCallCheck');

var _classCallCheck3 = _interopRequireDefault(_classCallCheck2);

var _slotFinder = require('slot-finder');

var _slotFinder2 = _interopRequireDefault(_slotFinder);

var _path = require('./utils/path');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var AppManagerServer = function AppManagerServer(config) {
  var _this = this;

  (0, _classCallCheck3.default)(this, AppManagerServer);

  this._getAppNameFromPath = function (path) {
    var appName = (0, _path.getAppNameFromPath)(_this._evaluatedPaths, path);

    if (!appName) {
      throw new Error('server.invalid_appPath');
    }

    return appName;
  };

  this._getAppFragments = function (appName) {
    var app = _this._apps[appName];

    if (!app) {
      throw new Error('server.app_not_found');
    }

    if (!Array.isArray(app.fragments)) {
      throw new Error('server.invalid_app');
    }

    var appFragments = app.fragments.map(function (fragmentName) {
      return _this._fragments[fragmentName];
    });

    var emptySlots = (0, _keys2.default)(_this._slots).reduce(function (empty, slotName) {
      return (0, _assign2.default)({}, empty, (0, _defineProperty3.default)({}, slotName, null));
    }, {});

    return (0, _slotFinder2.default)(emptySlots, appFragments);
  };

  this._getPathParams = function (appName, path) {
    return (0, _path.evalPathParams)(_this._evaluatedPaths[appName], path);
  };

  this.init = function () {
    var _ref = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee2(path) {
      for (var _len = arguments.length, getMarkupArgs = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
        getMarkupArgs[_key - 1] = arguments[_key];
      }

      var currentAppName, appFragments, state, getMarkupPromises, resolvedMarkup;
      return _regenerator2.default.wrap(function _callee2$(_context2) {
        while (1) {
          switch (_context2.prev = _context2.next) {
            case 0:
              currentAppName = _this._getAppNameFromPath(path);

              if (currentAppName) {
                _context2.next = 3;
                break;
              }

              throw new Error('init.no_route');

            case 3:
              appFragments = _this._getAppFragments(currentAppName);
              state = {
                status: null,
                event: null,
                path: path,
                params: _this._getPathParams(currentAppName, path) || {},
                prevApp: null,
                app: _this._apps[currentAppName]
              };
              getMarkupPromises = (0, _keys2.default)(appFragments).map(function () {
                var _ref2 = (0, _asyncToGenerator3.default)( /*#__PURE__*/_regenerator2.default.mark(function _callee(slotName) {
                  var fragmentName, fragment, markup;
                  return _regenerator2.default.wrap(function _callee$(_context) {
                    while (1) {
                      switch (_context.prev = _context.next) {
                        case 0:
                          fragmentName = appFragments[slotName];
                          fragment = _this._fragments[fragmentName];

                          if (!(typeof fragment.getMarkup === 'function')) {
                            _context.next = 8;
                            break;
                          }

                          _context.next = 5;
                          return fragment.getMarkup.apply(fragment, [state].concat((0, _toConsumableArray3.default)(getMarkupArgs)));

                        case 5:
                          _context.t0 = _context.sent;
                          _context.next = 9;
                          break;

                        case 8:
                          _context.t0 = '';

                        case 9:
                          markup = _context.t0;
                          return _context.abrupt('return', { slotName: slotName, markup: markup });

                        case 11:
                        case 'end':
                          return _context.stop();
                      }
                    }
                  }, _callee, _this);
                }));

                return function (_x2) {
                  return _ref2.apply(this, arguments);
                };
              }());
              _context2.next = 8;
              return _promise2.default.all(getMarkupPromises);

            case 8:
              resolvedMarkup = _context2.sent;
              return _context2.abrupt('return', resolvedMarkup.reduce(function (acc, _ref3) {
                var slotName = _ref3.slotName,
                    markup = _ref3.markup;
                return (0, _assign2.default)({}, acc, (0, _defineProperty3.default)({}, slotName, markup));
              }, {}));

            case 10:
            case 'end':
              return _context2.stop();
          }
        }
      }, _callee2, _this);
    }));

    return function (_x) {
      return _ref.apply(this, arguments);
    };
  }();

  var apps = config.apps,
      slots = config.slots,
      fragments = config.fragments;


  this._apps = apps;
  this._slots = slots;
  this._fragments = fragments;
  this._evaluatedPaths = (0, _path.evaluatePaths)(this._apps);
};

exports.default = AppManagerServer;